// This plugin comes from:
//
// http://www.keyframesandcode.com/resources/javascript/jQuery/demos/populate-demo.html

/* eslint-disable curly */
jQuery.fn.populate = function(obj, options) {

    // ------------------------------------------------------------------------------------------
    // JSON conversion function

    const arr = [];

    // convert
    function parseJSON(obj, path) {
        // prepare
        path = path || '';

        // iteration (objects / arrays)
        if (obj === undefined || obj === null) {
            // do nothing
        } else if (obj.constructor === Object) {
            for (const prop in obj) {
                const name = path + (path === '' ? prop : '[' + prop + ']');
                parseJSON(obj[prop], name);
            }
        } else if (obj.constructor === Array) {
            for (let i = 0; i < obj.length; i+=1) {
                let index = options.useIndices ? i : '';
                index = options.phpNaming ? '[' + index + ']' : index;
                const name = path + index;
                parseJSON(obj[i], name);
            }
            // assignment (values)
        } else {
            // if the element name hasn't yet been defined, create it as a single value
            if (arr[path] === undefined || arr[path] === null) {
                arr[path] = obj;
                // if the element name HAS been defined, but it's a single value, convert to an array and add the new value
            } else if (arr[path].constructor !== Array) {
                arr[path] = [arr[path], obj];
                // if the element name HAS been defined, and is already an array, push the single value on the end of the stack
            } else {
                arr[path].push(obj);
            }
        }
    }


    // ------------------------------------------------------------------------------------------
    // population functions

    function debug(str) {
        if (window.console && console.log) {
            console.log(str);
        }
    }

    // To get an escape selector (see http://learn.jquery.com/using-jquery-core/faq/how-do-i-select-an-element-by-an-id-that-has-characters-used-in-css-notation/)
    function jq(myid) {
        return "#" + myid.replace(/(:|\[|\]|\.)/g, "\\$1");
    }

    function getElementName(name) {
        if (!options.phpNaming) {
            name = name.replace(/\[\]$/, '');
        }
        return name;
    }

    function populateElement(parentElement, name, value) {
        const selector = options.identifier === 'id' ? '#' + name : '[' + options.identifier + '="' + name + '"]';
        const element = jQuery(selector, parentElement);
        if (!($.adapt && $.adapt.set(element, value))) {
            value = value.toString();
            value = value === "null" ? "" : value;
            element.html(value);
        }
    }

    function populateFormElement(form, name, value) {
        // check that the named element exists in the form
        name = getElementName(name); // handle non-php naming
        let element = form[name];

        // if the form element doesn't exist, check if there is a tag with that id
        if (element === undefined || element === null) {
            // look for the element
            element = jQuery(jq(name), form);
            if (element && element.length !== 0) {
                element.html(value);
                return true;
            }

            // nope, so exit
            if (options.debug) {
                debug('No such element as ' + name);
            }
            return false;
        }

        // now, place any single elements in an array.
        // this is so that the next bit of code (a loop) can treat them the
        // same as any array-elements passed, ie radiobutton or checkox arrays,
        // and the code will just work
        const elements = (element.type === undefined || element.type === null) && element.length ? element : new Array(element);

        // populate the element correctly
        for (let e = 0; e < elements.length; e+=1) {
            // grab the element
            element = elements[e];
            // skip undefined elements or function objects (IE only)
            if (!element || typeof element === 'undefined' || typeof element === 'function') continue;
            // skip elements with "skip" attribute set
            if (element.getAttribute && element.getAttribute("skip")) continue;

            // anything else, process
            switch (element.type || element.tagName) {
                case 'radio': {
                    // use the single value to check the radio button
                    element.checked = (element.value !== '' && value.toString() === element.value);
                    break;
                }
                // eslint-disable-next-line no-fallthrough
                case 'checkbox': {
                    // depends on the value.
                    // if it's an array, perform a sub loop
                    // if it's a value, just do the check
                    const values = value.constructor === Array ? value : [value];
                    for (let j = 0; j < values.length; j+=1) {
                        element.checked |= element.value === values[j].toString();
                    }
                    break;
                }
                case 'select-multiple': {
                    const values = value.constructor === Array ? value : [value];
                    for (let i = 0; i < element.options.length; i+=1) {
                        for (let j = 0; j < values.length; j+=1) {
                            element.options[i].selected |= element.options[i].value === values[j];
                        }
                    }
                    break;
                }
                case 'select':
                case 'select-one': {
                    // if the select box has been replaced by this
                    // selectbox plugin: https://gist.github.com/1139558
                    if ($('#' + element.id + '_container').length > 0) {
                        // Click the correct li to set the select box.
                        $('li#' + element.id + '_input_' + value).click()
                        // Otherwise just let it work the usual way.
                    } else {
                        element.value = value.toString() || value;
                    }
                    break;
                }
                case 'text':
                case 'button':
                case 'textarea':
                case 'submit':
                default: {
                    if (!($.adapt && $.adapt.set($(element), value))) {
                        value = (value === null || value === undefined) ? '' : value;
                        element.value = value;
                    }
                }
            }
        }
        return true;
    }

    // ------------------------------------------------------------------------------------------
    // options & setup

    // exit if no data object supplied
    if (obj === undefined || obj === null) {
        return this;
    }

    // options
    options = jQuery.extend({
        phpNaming: false,
        useIndices: false,
        resetForm: true,
        identifier: 'id',
        debug: false
    }, options);

    // ------------------------------------------------------------------------------------------
    // convert hierarchical JSON to flat array

    parseJSON(obj);

    // ------------------------------------------------------------------------------------------
    // main process function

    this.each(function() {
        // variables
        const tagName = this.tagName.toLowerCase();
        const method = tagName === 'form' ? populateFormElement : populateElement;

        // reset form?
        if (tagName === 'form' && options.resetForm) {
            this.reset();
        }

        // update elements
        for (const i in arr) {
            if (Object.prototype.hasOwnProperty.call(arr, i)) {
                method(this, i, arr[i]);
            }
        }
    });

    return this;
};
// vim: set ts=8 sts=4 sw=4 et sta :
